package main

import (
	"context"
	"flag"
	"fmt"
	"io"
	"net/http"
	"os"
	"time"

	"github.com/prometheus/client_golang/api"
	v1 "github.com/prometheus/client_golang/api/prometheus/v1"
	"github.com/prometheus/common/model"
)

func main() {
	var prometheusURL string
	var query string

	flag.StringVar(&prometheusURL, "h", "http://localhost:9090", "URL of Prometheus")
	flag.StringVar(&query, "q", "vector(0)", "PromQL query to run against prometheus")
	flag.Parse()

	client, PromErr := api.NewClient(api.Config{
		Address: prometheusURL,
	})
	if PromErr != nil {
		fmt.Printf("Error creating client: %v\n", PromErr)
		os.Exit(1)
	}

	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		result := queryPrometheus(client, query)
		switch result.Type() {
		case model.ValVector:
			valVector := result.(model.Vector)
			if len(valVector) == 1 {
				io.WriteString(w, fmt.Sprint(valVector[0].Value))
			} else {
				fmt.Printf("Unsupported multi-value return: %v\n", len(valVector))
			}
		default:
			fmt.Printf("Unsupported query type: %v\n", result.Type())
		}
	})

	http.HandleFunc("/healthz", func(w http.ResponseWriter, r *http.Request) {
		queryPrometheus(client, query)
	})

	httpErr := http.ListenAndServe(":8080", nil)
	if httpErr != nil {
		fmt.Printf("Error listening on port 8080: %v", httpErr)
		os.Exit(1)
	}
}

func queryPrometheus(client api.Client, query string) model.Value {
	v1api := v1.NewAPI(client)
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()
	result, warnings, err := v1api.Query(ctx, query, time.Now(), v1.WithTimeout(5*time.Second))
	if err != nil {
		fmt.Printf("Error querying Prometheus: %v\n", err)
		os.Exit(1)
	}
	if len(warnings) > 0 {
		fmt.Printf("Warnings: %v\n", warnings)
	}
	return result
}
